@@ -2,8 +2,8 @@
  * main.h: access to all program variables
  * Declaration and extern access to global program object.
  *****************************************************************************
- * Copyright (C) 1999, 2000 VideoLAN
- * $Id: main.h,v 1.33 2002/03/21 07:11:57 gbazin Exp $
+ * Copyright (C) 1999, 2000, 2001, 2002 VideoLAN
+ * $Id: main.h,v 1.52 2003/01/19 03:16:24 sam Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -23,60 +23,82 @@
  *****************************************************************************/
 
 /*****************************************************************************
- * main_t, p_main (global variable)
+ * libvlc_t (global variable)
  *****************************************************************************
- * This structure has an unique instance, declared in main and pointed by the
- * only global variable of the program. It should allow access to any variable
- * of the program, for user-interface purposes or more easier call of interface
- * and common functions (example: the intf_*Msg functions). Please avoid using
- * it when you can access the members you need in an other way. In fact, it
- * should only be used by interface thread.
+ * This structure has an unique instance, statically allocated in main and
+ * never accessed from the outside. It store once-initialized data such as
+ * the CPU capabilities or the global lock.
  *****************************************************************************/
-
-typedef struct main_s
+struct libvlc_t
 {
+    VLC_COMMON_MEMBERS
+
+    /* Initialization boolean */
+    vlc_bool_t             b_ready;
+
+    /* CPU extensions */
+    uint32_t               i_cpu;
+
+    /* Generic settings */
+    int                    i_verbose;                       /* info messages */
+    vlc_bool_t             b_color;                       /* color messages? */
+
+    /* Object structure data */
+    int                    i_counter;                      /* object counter */
+    int                    i_objects;              /* Attached objects count */
+    vlc_object_t **        pp_objects;               /* Array of all objects */
+
+    /* The message bank */
+    msg_bank_t             msg_bank;
+
+    /* The module bank */
+    module_bank_t *        p_module_bank;
+
+    /* Arch-specific variables */
+#if defined( SYS_BEOS )
+    vlc_object_t *         p_appthread;
+    char *                 psz_vlcpath;
+#elif defined( SYS_DARWIN )
+    char *                 psz_vlcpath;
+#elif defined( WIN32 ) && !defined( UNDER_CE )
+    SIGNALOBJECTANDWAIT    SignalObjectAndWait;
+    vlc_bool_t             b_fast_mutex;
+    int                    i_win9x_cv;
+#endif
+};
+
+/*****************************************************************************
+ * vlc_t, p_vlc
+ *****************************************************************************
+ * This structure is a LibVLC instance.
+ *****************************************************************************/
+struct vlc_t
+{
+    VLC_COMMON_MEMBERS
+
     /* Global properties */
     int                    i_argc;           /* command line arguments count */
     char **                ppsz_argv;              /* command line arguments */
-    char *                 psz_arg0;         /* program name (whithout path) */
     char *                 psz_homedir;             /* user's home directory */
 
-    u32                    i_cpu_capabilities;             /* CPU extensions */
-    int                    i_warning_level;        /* warning messages level */
-    boolean_t              b_stats;                  /* display statistics ? */
-
     /* Generic settings */
-    boolean_t              b_audio;             /* is audio output allowed ? */
-    boolean_t              b_video;             /* is video output allowed ? */
-    boolean_t              b_stereo;
     mtime_t                i_desync;   /* relative desync of the audio ouput */
 
     /* Fast memcpy plugin used */
     module_t *             p_memcpy_module;
+#if defined( UNDER_CE )
+    void* ( __cdecl *pf_memcpy ) ( void *, const void *, size_t );
+    void* ( __cdecl *pf_memset ) ( void *, int, size_t );
+#else
     void* ( *pf_memcpy ) ( void *, const void *, size_t );
-    void* ( *pf_memset ) ( void *, int, size_t );    /* FIXME: unimplemented */
+    void* ( *pf_memset ) ( void *, int, size_t );
+#endif
 
-    /* Unique threads */
-    p_intf_thread_t        p_intf;                  /* main interface thread */
-
-    /* Shared data - these structures are accessed directly from p_main by
+    /* Shared data - these structures are accessed directly from p_vlc by
      * several modules */
-    p_playlist_t           p_playlist;                           /* playlist */
-    p_intf_msg_t           p_msg;                 /* messages interface data */
-    p_input_channel_t      p_channel;                /* channel library data */
+    input_channel_t *      p_channel;                /* channel library data */
 
     /* Locks */
     vlc_mutex_t            config_lock;          /* lock for the config file */
-} main_t;
+};
 
-#ifndef PLUGIN
-extern main_t *p_main;
-#else
-#   define p_main (p_symbols->p_main)
-#endif
-
-/*****************************************************************************
- * Fast memory operation module
- *****************************************************************************/
-#define FAST_MEMCPY p_main->pf_memcpy
-#define FAST_MEMSET p_main->pf_memset
