@@ -2,7 +2,7 @@
  * input_programs.c: es_descriptor_t, pgrm_descriptor_t management
  *****************************************************************************
  * Copyright (C) 1999-2002 VideoLAN
- * $Id: input_programs.c,v 1.88 2002/05/17 00:58:14 sam Exp $
+ * $Id: input_programs.c,v 1.101 2003/01/31 11:23:37 massiot Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -10,7 +10,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -26,9 +26,8 @@
  *****************************************************************************/
 #include <stdlib.h>
 #include <string.h>                                    /* memcpy(), memset() */
-#include <sys/types.h>                                              /* off_t */
 
-#include <videolan/vlc.h>
+#include <vlc/vlc.h>
 
 #include "stream_control.h"
 #include "input_ext-intf.h"
@@ -58,12 +57,12 @@ int input_InitStream( input_thread_t * p_input, size_t i_data_len )
     p_input->stream.pp_programs = NULL;
     p_input->stream.p_selected_program = NULL;
     p_input->stream.p_new_program = NULL;
-    
+
     if( i_data_len )
     {
         if ( (p_input->stream.p_demux_data = malloc( i_data_len )) == NULL )
         {
-            intf_ErrMsg( "Unable to allocate memory in input_InitStream");
+            msg_Err( p_input, "out of memory" );
             return 1;
         }
         memset( p_input->stream.p_demux_data, 0, i_data_len );
@@ -104,7 +103,7 @@ void input_EndStream( input_thread_t * p_input )
     {
         free( p_input->stream.pp_selected_es );
     }
-    
+
     if( p_input->stream.p_demux_data != NULL )
     {
         free( p_input->stream.p_demux_data );
@@ -114,9 +113,10 @@ void input_EndStream( input_thread_t * p_input )
 /*****************************************************************************
  * input_FindProgram: returns a pointer to a program described by its ID
  *****************************************************************************/
-pgrm_descriptor_t * input_FindProgram( input_thread_t * p_input, u16 i_pgrm_id )
+pgrm_descriptor_t * input_FindProgram( input_thread_t * p_input,
+                                       uint16_t i_pgrm_id )
 {
-    int     i;
+    unsigned int i;
 
     for( i = 0; i < p_input->stream.i_pgrm_number; i++ )
     {
@@ -138,59 +138,48 @@ pgrm_descriptor_t * input_AddProgram( input_thread_t * p_input,
                                       u16 i_pgrm_id, size_t i_data_len )
 {
     /* Where to add the pgrm */
-    int i_pgrm_index = p_input->stream.i_pgrm_number;
+    pgrm_descriptor_t * p_pgrm = malloc( sizeof(pgrm_descriptor_t) );
 
-    /* Add an entry to the list of program associated with the stream */
-    p_input->stream.i_pgrm_number++;
-    p_input->stream.pp_programs = realloc( p_input->stream.pp_programs,
-                                           p_input->stream.i_pgrm_number
-                                           * sizeof(pgrm_descriptor_t *) );
-    if( p_input->stream.pp_programs == NULL )
+    if( p_pgrm == NULL )
     {
-        intf_ErrMsg( "Unable to realloc memory in input_AddProgram" );
-        return( NULL );
+        msg_Err( p_input, "out of memory" );
+        return NULL;
     }
 
-    /* Allocate the structure to store this description */
-    p_input->stream.pp_programs[i_pgrm_index] =
-                                        malloc( sizeof(pgrm_descriptor_t) );
-    if( p_input->stream.pp_programs[i_pgrm_index] == NULL )
-    {
-        intf_ErrMsg( "Unable to allocate memory in input_AddProgram" );
-        return( NULL );
-    }
-    
     /* Init this entry */
-    p_input->stream.pp_programs[i_pgrm_index]->i_number = i_pgrm_id;
-    p_input->stream.pp_programs[i_pgrm_index]->b_is_ok = 0;
-    p_input->stream.pp_programs[i_pgrm_index]->i_version = 0;
+    p_pgrm->i_number = i_pgrm_id;
+    p_pgrm->b_is_ok = 0;
+    p_pgrm->i_version = 0;
 
-    p_input->stream.pp_programs[i_pgrm_index]->i_es_number = 0;
-    p_input->stream.pp_programs[i_pgrm_index]->pp_es = NULL;
+    p_pgrm->i_es_number = 0;
+    p_pgrm->pp_es = NULL;
 
-    input_ClockInit( p_input->stream.pp_programs[i_pgrm_index] );
+    input_ClockInit( p_pgrm );
 
-    p_input->stream.pp_programs[i_pgrm_index]->i_synchro_state
-                                                = SYNCHRO_START;
+    p_pgrm->i_synchro_state = SYNCHRO_START;
 
     if( i_data_len )
     {
-        p_input->stream.pp_programs[i_pgrm_index]->p_demux_data =
-            malloc( i_data_len );
-        if( p_input->stream.pp_programs[i_pgrm_index]->p_demux_data == NULL )
+        p_pgrm->p_demux_data = malloc( i_data_len );
+        if( p_pgrm->p_demux_data == NULL )
         {
-            intf_ErrMsg( "Unable to allocate memory in input_AddProgram" );
-            return( NULL );
+            msg_Err( p_input, "out of memory" );
+            return NULL;
         }
-        memset( p_input->stream.pp_programs[i_pgrm_index]->p_demux_data, 0,
-                i_data_len );
+        memset( p_pgrm->p_demux_data, 0, i_data_len );
     }
     else
     {
-        p_input->stream.pp_programs[i_pgrm_index]->p_demux_data = NULL;
+        p_pgrm->p_demux_data = NULL;
     }
 
-    return p_input->stream.pp_programs[i_pgrm_index];
+    /* Add an entry to the list of program associated with the stream */
+    INSERT_ELEM( p_input->stream.pp_programs,
+                 p_input->stream.i_pgrm_number,
+                 p_input->stream.i_pgrm_number,
+                 p_pgrm );
+
+    return p_pgrm;
 }
 
 /*****************************************************************************
@@ -200,7 +189,7 @@ pgrm_descriptor_t * input_AddProgram( input_thread_t * p_input,
  *****************************************************************************/
 void input_DelProgram( input_thread_t * p_input, pgrm_descriptor_t * p_pgrm )
 {
-    int i_pgrm_index;
+    unsigned int i_pgrm_index;
 
     /* Find the program in the programs table */
     for( i_pgrm_index = 0; i_pgrm_index < p_input->stream.i_pgrm_number;
@@ -213,7 +202,7 @@ void input_DelProgram( input_thread_t * p_input, pgrm_descriptor_t * p_pgrm )
     /* If the program wasn't found, do nothing */
     if( i_pgrm_index == p_input->stream.i_pgrm_number )
     {
-        intf_ErrMsg( "input error: program does not belong to this input" );
+        msg_Err( p_input, "program does not belong to this input" );
         return;
     }
 
@@ -230,26 +219,9 @@ void input_DelProgram( input_thread_t * p_input, pgrm_descriptor_t * p_pgrm )
     }
 
     /* Remove this program from the stream's list of programs */
-    p_input->stream.i_pgrm_number--;
-
-    p_input->stream.pp_programs[i_pgrm_index] =
-        p_input->stream.pp_programs[p_input->stream.i_pgrm_number];
-    if( p_input->stream.i_pgrm_number ) 
-    {
-        p_input->stream.pp_programs = realloc( p_input->stream.pp_programs,
-                                               p_input->stream.i_pgrm_number
-                                               * sizeof(pgrm_descriptor_t *) );
-        if( p_input->stream.pp_programs == NULL )
-        {
-            intf_ErrMsg( "input error: unable to realloc program list"
-                         " in input_DelProgram" );
-        }
-    }
-    else
-    {
-        free( p_input->stream.pp_programs );
-        p_input->stream.pp_programs = NULL;
-    }
+    REMOVE_ELEM( p_input->stream.pp_programs,
+                 p_input->stream.i_pgrm_number,
+                 i_pgrm_index );
 
     /* Free the description of this program */
     free( p_pgrm );
@@ -263,38 +235,30 @@ void input_DelProgram( input_thread_t * p_input, pgrm_descriptor_t * p_pgrm )
 input_area_t * input_AddArea( input_thread_t * p_input )
 {
     /* Where to add the pgrm */
-    int i_area_index = p_input->stream.i_area_nb;
+    input_area_t * p_area = malloc( sizeof(input_area_t) );
+
+    if( p_area == NULL )
+    {
+        msg_Err( p_input, "out of memory" );
+        return NULL;
+    }
+
+    /* Init this entry */
+    p_area->i_id = 0;
+    p_area->i_start = 0;
+    p_area->i_size = 0;
+    p_area->i_tell = 0;
+    p_area->i_seek = NO_SEEK;
+    p_area->i_part_nb = 1;
+    p_area->i_part= 0;
 
     /* Add an entry to the list of program associated with the stream */
-    p_input->stream.i_area_nb++;
-    p_input->stream.pp_areas = realloc( p_input->stream.pp_areas,
-                                        p_input->stream.i_area_nb
-                                        * sizeof(input_area_t *) );
-    if( p_input->stream.pp_areas == NULL )
-    {
-        intf_ErrMsg( "Unable to realloc memory in input_AddArea" );
-        return( NULL );
-    }
+    INSERT_ELEM( p_input->stream.pp_areas,
+                 p_input->stream.i_area_nb,
+                 p_input->stream.i_area_nb,
+                 p_area );
 
-    /* Allocate the structure to store this description */
-    p_input->stream.pp_areas[i_area_index] =
-                                        malloc( sizeof(input_area_t) );
-    if( p_input->stream.pp_areas[i_area_index] == NULL )
-    {
-        intf_ErrMsg( "Unable to allocate memory in input_AddArea" );
-        return( NULL );
-    }
-    
-    /* Init this entry */
-    p_input->stream.pp_areas[i_area_index]->i_id = 0;
-    p_input->stream.pp_areas[i_area_index]->i_start = 0;
-    p_input->stream.pp_areas[i_area_index]->i_size = 0;
-    p_input->stream.pp_areas[i_area_index]->i_tell = 0;
-    p_input->stream.pp_areas[i_area_index]->i_seek = NO_SEEK;
-    p_input->stream.pp_areas[i_area_index]->i_part_nb = 1;
-    p_input->stream.pp_areas[i_area_index]->i_part= 0;
-
-    return p_input->stream.pp_areas[i_area_index];
+    return p_area;
 }
 
 /*****************************************************************************
@@ -302,7 +266,7 @@ input_area_t * input_AddArea( input_thread_t * p_input )
  *****************************************************************************/
 int input_SetProgram( input_thread_t * p_input, pgrm_descriptor_t * p_new_prg )
 {
-    int i_es_index;
+    unsigned int i_es_index;
     int i_required_audio_es;
     int i_required_spu_es;
     int i_audio_es = 0;
@@ -324,10 +288,10 @@ int input_SetProgram( input_thread_t * p_input, pgrm_descriptor_t * p_new_prg )
         }
     }
     /* Get the number of the required audio stream */
-    if( p_main->b_audio )
+    if( config_GetInt( p_input, "audio" ) )
     {
         /* Default is the first one */
-        i_required_audio_es = config_GetIntVariable( "audio-channel" );
+        i_required_audio_es = config_GetInt( p_input, "audio-channel" );
         if( i_required_audio_es < 0 )
         {
             i_required_audio_es = 1;
@@ -339,10 +303,10 @@ int input_SetProgram( input_thread_t * p_input, pgrm_descriptor_t * p_new_prg )
     }
 
     /* Same thing for subtitles */
-    if( p_main->b_video )
+    if( config_GetInt( p_input, "video" ) )
     {
         /* for spu, default is none */
-        i_required_spu_es = config_GetIntVariable( "spu-channel" );
+        i_required_spu_es = config_GetInt( p_input, "spu-channel" );
         if( i_required_spu_es < 0 )
         {
             i_required_spu_es = 0;
@@ -353,21 +317,21 @@ int input_SetProgram( input_thread_t * p_input, pgrm_descriptor_t * p_new_prg )
         i_required_spu_es = 0;
     }
 
-    for (i_es_index = 0 ; i_es_index < p_new_prg->i_es_number ; i_es_index ++ )
+    for( i_es_index = 0 ; i_es_index < p_new_prg->i_es_number ; i_es_index ++ )
     {
         switch( p_new_prg->pp_es[i_es_index]->i_cat )
         {
             case VIDEO_ES:
-                intf_WarnMsg( 4, "Selecting ES %x",
-                            p_new_prg->pp_es[i_es_index]->i_id );
+                msg_Dbg( p_input, "selecting ES %x",
+                         p_new_prg->pp_es[i_es_index]->i_id );
                 input_SelectES( p_input, p_new_prg->pp_es[i_es_index] );
                 break;
             case AUDIO_ES:
                 i_audio_es += 1;
                 if( i_audio_es <= i_required_audio_es )
                 {
-                    intf_WarnMsg( 4, "Selecting ES %x",
-                                p_new_prg->pp_es[i_es_index]->i_id );
+                    msg_Dbg( p_input, "selecting ES %x",
+                             p_new_prg->pp_es[i_es_index]->i_id );
                     input_SelectES( p_input, p_new_prg->pp_es[i_es_index]);
                 }
                 break;
@@ -376,14 +340,14 @@ int input_SetProgram( input_thread_t * p_input, pgrm_descriptor_t * p_new_prg )
                 i_spu_es += 1;
                 if( i_spu_es <= i_required_spu_es )
                 {
-                    intf_WarnMsg( 4, "Selecting ES %x",
-                                p_new_prg->pp_es[i_es_index]->i_id );
+                    msg_Dbg( p_input, "selecting ES %x",
+                             p_new_prg->pp_es[i_es_index]->i_id );
                     input_SelectES( p_input, p_new_prg->pp_es[i_es_index] );
                 }
             break;
             default :
-                intf_WarnMsg( 2, "ES %x has unknown type",
-                            p_new_prg->pp_es[i_es_index]->i_id );
+                msg_Dbg( p_input, "ES %x has unknown type",
+                         p_new_prg->pp_es[i_es_index]->i_id );
                 break;
         }
 
@@ -403,7 +367,7 @@ int input_SetProgram( input_thread_t * p_input, pgrm_descriptor_t * p_new_prg )
  *****************************************************************************/
 void input_DelArea( input_thread_t * p_input, input_area_t * p_area )
 {
-    int i_area_index;
+    unsigned int i_area_index;
 
     /* Find the area in the areas table */
     for( i_area_index = 0; i_area_index < p_input->stream.i_area_nb;
@@ -416,32 +380,14 @@ void input_DelArea( input_thread_t * p_input, input_area_t * p_area )
     /* If the area wasn't found, do nothing */
     if( i_area_index == p_input->stream.i_area_nb )
     {
-        intf_ErrMsg( "input error: area does not belong to this input" );
+        msg_Err( p_input, "area does not belong to this input" );
         return;
     }
 
     /* Remove this area from the stream's list of areas */
-    p_input->stream.i_area_nb--;
-
-    p_input->stream.pp_areas[i_area_index] =
-        p_input->stream.pp_areas[p_input->stream.i_area_nb];
-    if( p_input->stream.i_area_nb )
-    {
-        p_input->stream.pp_areas = realloc( p_input->stream.pp_areas,
-                                            p_input->stream.i_area_nb
-                                            * sizeof(input_area_t *) );
-
-        if( p_input->stream.pp_areas == NULL )
-        {
-            intf_ErrMsg( "input error: unable to realloc area list"
-                         " in input_DelArea" );
-        }
-    }
-    else
-    {
-        free( p_input->stream.pp_areas );
-        p_input->stream.pp_areas = NULL;
-    }
+    REMOVE_ELEM( p_input->stream.pp_areas,
+                 p_input->stream.i_area_nb,
+                 i_area_index );
 
     /* Free the description of this area */
     free( p_area );
@@ -451,9 +397,9 @@ void input_DelArea( input_thread_t * p_input, input_area_t * p_area )
 /*****************************************************************************
  * input_FindES: returns a pointer to an ES described by its ID
  *****************************************************************************/
-es_descriptor_t * input_FindES( input_thread_t * p_input, u16 i_es_id )
+es_descriptor_t * input_FindES( input_thread_t * p_input, uint16_t i_es_id )
 {
-    int     i;
+    unsigned int i;
 
     for( i = 0; i < p_input->stream.i_es_number; i++ )
     {
@@ -482,36 +428,31 @@ es_descriptor_t * input_AddES( input_thread_t * p_input,
     p_es = (es_descriptor_t *)malloc( sizeof(es_descriptor_t) );
     if( p_es == NULL )
     {
-        intf_ErrMsg( "Unable to allocate memory in input_AddES" );
+        msg_Err( p_input, "out of memory" );
         return( NULL);
     }
-    p_input->stream.i_es_number++;
-    p_input->stream.pp_es = realloc( p_input->stream.pp_es,
-                                     p_input->stream.i_es_number
-                                      * sizeof(es_descriptor_t *) );
-    if( p_input->stream.pp_es == NULL )
-    {
-        intf_ErrMsg( "Unable to realloc memory in input_AddES" );
-        return( NULL );
-    }
 
-    p_input->stream.pp_es[p_input->stream.i_es_number - 1] = p_es;
+    INSERT_ELEM( p_input->stream.pp_es,
+                 p_input->stream.i_es_number,
+                 p_input->stream.i_es_number,
+                 p_es );
 
     /* Init its values */
     p_es->i_id = i_es_id;
     p_es->psz_desc[0] = '\0';
     p_es->p_pes = NULL;
     p_es->p_decoder_fifo = NULL;
-    p_es->b_audio = 0;
     p_es->i_cat = UNKNOWN_ES;
     p_es->i_demux_fd = 0;
+    p_es->c_packets = 0;
+    p_es->c_invalid_packets = 0;
 
     if( i_data_len )
     {
         p_es->p_demux_data = malloc( i_data_len );
         if( p_es->p_demux_data == NULL )
         {
-            intf_ErrMsg( "Unable to allocate memory in input_AddES" );
+            msg_Err( p_input, "out of memory" );
             return( NULL );
         }
         memset( p_es->p_demux_data, 0, i_data_len );
@@ -520,21 +461,16 @@ es_descriptor_t * input_AddES( input_thread_t * p_input,
     {
         p_es->p_demux_data = NULL;
     }
+    p_es->p_waveformatex     = NULL;
+    p_es->p_bitmapinfoheader = NULL;
 
     /* Add this ES to the program definition if one is given */
     if( p_pgrm )
     {
-        p_pgrm->i_es_number++;
-        p_pgrm->pp_es = realloc( p_pgrm->pp_es,
-                                 p_pgrm->i_es_number
-                                  * sizeof(es_descriptor_t *) );
-        if( p_pgrm->pp_es == NULL )
-        {
-            intf_ErrMsg( "Unable to realloc memory in input_AddES" );
-            return( NULL );
-        }
-
-        p_pgrm->pp_es[p_pgrm->i_es_number - 1] = p_es;
+        INSERT_ELEM( p_pgrm->pp_es,
+                     p_pgrm->i_es_number,
+                     p_pgrm->i_es_number,
+                     p_es );
         p_es->p_pgrm = p_pgrm;
     }
     else
@@ -550,7 +486,7 @@ es_descriptor_t * input_AddES( input_thread_t * p_input,
  *****************************************************************************/
 void input_DelES( input_thread_t * p_input, es_descriptor_t * p_es )
 {
-    int                     i_index, i_es_index;
+    unsigned int            i_index, i_es_index;
     pgrm_descriptor_t *     p_pgrm;
 
     /* Find the ES in the ES table */
@@ -564,7 +500,7 @@ void input_DelES( input_thread_t * p_input, es_descriptor_t * p_es )
     /* If the ES wasn't found, do nothing */
     if( i_es_index == p_input->stream.i_es_number )
     {
-        intf_ErrMsg( "input error: ES does not belong to this input" );
+        msg_Err( p_input, "ES does not belong to this input" );
         return;
     }
 
@@ -584,24 +520,9 @@ void input_DelES( input_thread_t * p_input, es_descriptor_t * p_es )
         {
             if( p_pgrm->pp_es[i_index] == p_es )
             {
-                p_pgrm->i_es_number--;
-                p_pgrm->pp_es[i_index] = p_pgrm->pp_es[p_pgrm->i_es_number];
-                if( p_pgrm->i_es_number )
-                {
-                    p_pgrm->pp_es = realloc( p_pgrm->pp_es,
-                                             p_pgrm->i_es_number
-                                              * sizeof(es_descriptor_t *));
-                    if( p_pgrm->pp_es == NULL )
-                    {
-                        intf_ErrMsg( "Unable to realloc memory in "
-                                     "input_DelES" );
-                    }
-                }
-                else
-                {
-                    free( p_pgrm->pp_es );
-                    p_pgrm->pp_es = NULL;
-                }
+                REMOVE_ELEM( p_pgrm->pp_es,
+                             p_pgrm->i_es_number,
+                             i_index );
                 break;
             }
         }
@@ -612,27 +533,28 @@ void input_DelES( input_thread_t * p_input, es_descriptor_t * p_es )
     {
         free( p_es->p_demux_data );
     }
+    if( p_es->p_waveformatex )
+    {
+        free( p_es->p_waveformatex );
+    }
+    if( p_es->p_bitmapinfoheader )
+    {
+        free( p_es->p_bitmapinfoheader );
+    }
+
+    /* Find the ES in the ES table */
+    for( i_es_index = 0; i_es_index < p_input->stream.i_es_number;
+         i_es_index++ )
+    {
+        if( p_input->stream.pp_es[i_es_index] == p_es )
+            break;
+    }
 
     /* Remove this ES from the stream's list of ES */
-    p_input->stream.i_es_number--;
-    p_input->stream.pp_es[i_es_index] =
-                    p_input->stream.pp_es[p_input->stream.i_es_number];
-    if( p_input->stream.i_es_number )
-    {
-        p_input->stream.pp_es = realloc( p_input->stream.pp_es,
-                                         p_input->stream.i_es_number
-                                          * sizeof(es_descriptor_t *));
-        if( p_input->stream.pp_es == NULL )
-        {
-            intf_ErrMsg( "Unable to realloc memory in input_DelES" );
-        }
-    }
-    else
-    {
-        free( p_input->stream.pp_es );
-        p_input->stream.pp_es = NULL;
-    }
-    
+    REMOVE_ELEM( p_input->stream.pp_es,
+                 p_input->stream.i_es_number,
+                 i_es_index );
+
     /* Free the ES */
     free( p_es );
 }
@@ -647,65 +569,45 @@ int input_SelectES( input_thread_t * p_input, es_descriptor_t * p_es )
 {
     if( p_es == NULL )
     {
-        intf_ErrMsg( "input error: nothing to do in input_SelectES" );
+        msg_Err( p_input, "nothing to do in input_SelectES" );
         return -1;
     }
 
-    intf_WarnMsg( 4, "input: selecting ES 0x%x", p_es->i_id );
+    if( ((p_es->i_cat == VIDEO_ES) || (p_es->i_cat == SPU_ES))
+        && !config_GetInt( p_input, "video" ) )
+    {
+        msg_Dbg( p_input,
+                 "video is disabled, not selecting ES 0x%x", p_es->i_id );
+        return -1;
+    }
+
+    if( (p_es->i_cat == AUDIO_ES) && !config_GetInt( p_input, "audio" ) )
+    {
+        msg_Dbg( p_input,
+                 "audio is disabled, not selecting ES 0x%x", p_es->i_id );
+        return -1;
+    }
+
+    msg_Dbg( p_input, "selecting ES 0x%x", p_es->i_id );
 
     if( p_es->p_decoder_fifo != NULL )
     {
-        intf_ErrMsg( "ES 0x%x is already selected", p_es->i_id );
-        return( -1 );
+        msg_Err( p_input, "ES 0x%x is already selected", p_es->i_id );
+        return -1;
     }
 
-    p_es->thread_id = 0;
+    /* Release the lock, not to block the input thread during
+     * the creation of the thread. */
+    vlc_mutex_unlock( &p_input->stream.stream_lock );
+    p_es->p_decoder_fifo = input_RunDecoder( p_input, p_es );
+    vlc_mutex_lock( &p_input->stream.stream_lock );
 
-    switch( p_es->i_type )
+    if( p_es->p_decoder_fifo == NULL )
     {
-    case AC3_AUDIO_ES:
-    case MPEG1_AUDIO_ES:
-    case MPEG2_AUDIO_ES:
-    case LPCM_AUDIO_ES:
-        if( p_main->b_audio )
-        {
-            /* Release the lock, not to block the input thread during
-             * the creation of the thread. */
-            vlc_mutex_unlock( &p_input->stream.stream_lock );
-            p_es->thread_id = input_RunDecoder( p_input, p_es );
-            vlc_mutex_lock( &p_input->stream.stream_lock );
-        }
-        break;
-
-    case MPEG1_VIDEO_ES:
-    case MPEG2_VIDEO_ES:
-    case MPEG4_VIDEO_ES:
-    case MSMPEG4v1_VIDEO_ES:
-    case MSMPEG4v2_VIDEO_ES:
-    case MSMPEG4v3_VIDEO_ES:
-    case DVD_SPU_ES:
-        if( p_main->b_video )
-        {
-            /* Release the lock, not to block the input thread during
-             * the creation of the thread. */
-            vlc_mutex_unlock( &p_input->stream.stream_lock );
-            p_es->thread_id = input_RunDecoder( p_input, p_es );
-            vlc_mutex_lock( &p_input->stream.stream_lock );
-        }
-        break;
-
-    default:
-        intf_ErrMsg( "Unknown stream type 0x%x", p_es->i_type );
-        return( -1 );
-        break;
+        return -1;
     }
 
-    if( p_es->thread_id == 0 )
-    {
-        return( -1 );
-    }
-
-    return( 0 );
+    return 0;
 }
 
 /*****************************************************************************
@@ -714,19 +616,19 @@ int input_SelectES( input_thread_t * p_input, es_descriptor_t * p_es )
 int input_UnselectES( input_thread_t * p_input, es_descriptor_t * p_es )
 {
 
-    int     i_index = 0;
+    unsigned int i_index = 0;
 
     if( p_es == NULL )
     {
-        intf_ErrMsg( "Nothing to do in input_UnselectES" );
+        msg_Err( p_input, "nothing to do in input_UnselectES" );
         return -1;
     }
 
-    intf_WarnMsg( 4, "input: unselecting ES 0x%x", p_es->i_id );
+    msg_Dbg( p_input, "unselecting ES 0x%x", p_es->i_id );
 
     if( p_es->p_decoder_fifo == NULL )
     {
-        intf_ErrMsg( "ES 0x%x is not selected", p_es->i_id );
+        msg_Err( p_input, "ES 0x%x is not selected", p_es->i_id );
         return( -1 );
     }
 
@@ -736,36 +638,23 @@ int input_UnselectES( input_thread_t * p_input, es_descriptor_t * p_es )
     if( ( p_es->p_decoder_fifo == NULL ) &&
         ( p_input->stream.i_selected_es_number > 0 ) )
     {
-        p_input->stream.i_selected_es_number--;
-
-        while( ( i_index < p_input->stream.i_selected_es_number ) &&
+        while( ( i_index < p_input->stream.i_selected_es_number - 1 ) &&
                ( p_input->stream.pp_selected_es[i_index] != p_es ) )
         {
             i_index++;
         }
 
-        p_input->stream.pp_selected_es[i_index] = 
-          p_input->stream.pp_selected_es[p_input->stream.i_selected_es_number];
+        /* XXX: no need to memmove, we have unsorted data */
+        REMOVE_ELEM( p_input->stream.pp_selected_es,
+                     p_input->stream.i_selected_es_number,
+                     i_index );
 
-        if( p_input->stream.i_selected_es_number )
+        if( p_input->stream.i_selected_es_number == 0 )
         {
-            p_input->stream.pp_selected_es = realloc(
-                                           p_input->stream.pp_selected_es,
-                                           p_input->stream.i_selected_es_number
-                                           * sizeof(es_descriptor_t *) );
-            if( p_input->stream.pp_selected_es == NULL )
-            {
-                intf_ErrMsg( "Unable to realloc memory in input_UnselectES" );
-                return( -1 );
-            }
-        }
-        else
-        {
-            free( p_input->stream.pp_selected_es );   
-            p_input->stream.pp_selected_es = NULL;
-            intf_WarnMsg( 4, "input: no more selected ES in input_UnselectES" );            return( 1 );
+            msg_Dbg( p_input, "no more selected ES" );
+            return 1;
         }
     }
 
-    return( 0 );
+    return 0;
 }
