@@ -2,7 +2,7 @@
  * netutils.c: various network functions
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: netutils.c,v 1.66.2.2 2002/06/03 00:28:08 sam Exp $
+ * $Id: netutils.c,v 1.79 2003/01/27 21:44:39 ipkiss Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Benoit Steiner <benny@via.ecp.fr>
@@ -10,6 +10,7 @@
  *          Xavier Marchesini <xav@alarue.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
  *          Samuel Hocevar <sam@via.ecp.fr>
+ *          Jon Lech Johansen <jon-vl@nanocrew.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,14 +31,17 @@
  * Preamble
  *****************************************************************************/
 #include <stdlib.h>                             /* free(), realloc(), atoi() */
-#include <errno.h>                                                /* errno() */
 #include <string.h>                                              /* memset() */
 
-#include <videolan/vlc.h>
+#include <vlc/vlc.h>
+
+#ifdef HAVE_ERRNO_H
+#   include <errno.h>                                             /* errno() */
+#endif
 
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>                                      /* gethostname() */
-#elif defined( _MSC_VER ) && defined( _WIN32 )
+#elif defined( _MSC_VER ) && defined( _WIN32 ) && !defined( UNDER_CE )
 #   include <io.h>
 #endif
 
@@ -45,7 +49,9 @@
 #include <sys/time.h>                                        /* gettimeofday */
 #endif
 
-#ifdef WIN32
+#if defined( UNDER_CE )
+    /* No real network support (FIXME: use winsock1?) */
+#elif defined( WIN32 )
 #   include <winsock2.h>
 #else
 #   include <netdb.h>                                         /* hostent ... */
@@ -60,10 +66,17 @@
 #include <sys/ioctl.h>                                            /* ioctl() */
 #endif
 
-#if defined( WIN32 )                    /* tools to get the MAC adress from  */
-#include <windows.h>                    /* the interface under Windows       */
-#include <stdio.h>
-#include <nb30.h>
+#ifdef SYS_DARWIN
+#include <IOKit/IOKitLib.h>
+#include <IOKit/network/IOEthernetInterface.h>
+#include <IOKit/network/IOEthernetController.h>
+#endif
+
+#if defined( WIN32 ) && !defined( UNDER_CE )
+    /* tools to get the MAC adress from the interface under Windows */
+#   include <windows.h>
+#   include <stdio.h>
+#   include <nb30.h>
 #endif
 
 #ifdef HAVE_NET_IF_H
@@ -75,11 +88,8 @@
 #endif
 
 #include "netutils.h"
+#include "vlc_playlist.h"
 
-#include "stream_control.h"
-#include "input_ext-intf.h"
-
-#include "intf_playlist.h"
 #include "network.h"
 
 /*****************************************************************************
@@ -90,17 +100,20 @@
  * as it depends on the VideoLAN channel server, which isn't frozen for
  * the time being.
  *****************************************************************************/
-typedef struct input_channel_s
+struct input_channel_t
 {
     int         i_channel;                         /* current channel number */
     mtime_t     last_change;                             /* last change date */
-} input_channel_t;
+};
 
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static int GetMacAddress   ( int i_fd, char *psz_mac );
-#ifdef WIN32
+static int GetMacAddress   ( vlc_object_t *, int i_fd, char *psz_mac );
+#ifdef SYS_DARWIN
+static int GetNetIntfCtrl  ( const char *psz_interface, 
+                             io_object_t *ctrl_service );
+#elif defined( WIN32 ) && !defined( UNDER_CE )
 static int GetAdapterInfo  ( int i_adapter, char *psz_string );
 #endif
 
@@ -111,26 +124,26 @@ static int GetAdapterInfo  ( int i_adapter, char *psz_string );
  * once before any input thread is created or any call to other
  * input_Channel*() function is attempted.
  *****************************************************************************/
-int network_ChannelCreate( void )
+int __network_ChannelCreate( vlc_object_t *p_this )
 {
-#if !defined( SYS_LINUX ) && !defined( WIN32 )
-    intf_ErrMsg( "channel warning: VLAN-based channels are not supported"
-                 " under this architecture" );
+#if !defined( SYS_LINUX ) && !( defined( WIN32 ) && !defined( UNDER_CE ) )
+    msg_Warn( p_this, "VLAN-based channels are not supported "
+                      "on this architecture" );
 #endif
 
     /* Allocate structure */
-    p_main->p_channel = malloc( sizeof( input_channel_t ) );
-    if( p_main->p_channel == NULL )
+    p_this->p_vlc->p_channel = malloc( sizeof( input_channel_t ) );
+    if( p_this->p_vlc->p_channel == NULL )
     {
-        intf_ErrMsg( "network error: could not create channel bank" );
+        msg_Err( p_this, "out of memory" );
         return( -1 );
     }
 
     /* Initialize structure */
-    p_main->p_channel->i_channel   = 0;
-    p_main->p_channel->last_change = 0;
+    p_this->p_vlc->p_channel->i_channel   = 0;
+    p_this->p_vlc->p_channel->last_change = 0;
 
-    intf_WarnMsg( 2, "network: channels initialized" );
+    msg_Dbg( p_this, "channels initialized" );
     return( 0 );
 }
 
@@ -145,14 +158,15 @@ int network_ChannelCreate( void )
  * should be unlocked using input_ChannelLeave().
  * Non 0 will be returned in case of error.
  *****************************************************************************/
-int network_ChannelJoin( int i_channel )
+int __network_ChannelJoin( vlc_object_t *p_this, int i_channel )
 {
 #define VLCS_VERSION 13
 #define MESSAGE_LENGTH 256
 
-    struct module_s *   p_network;
-    char *              psz_network = NULL;
-    network_socket_t    socket_desc;
+#if defined( SYS_LINUX ) || ( defined( WIN32 ) && !defined( UNDER_CE ) )
+    module_t *       p_network;
+    char *           psz_network = NULL;
+    network_socket_t socket_desc;
     char psz_mess[ MESSAGE_LENGTH ];
     char psz_mac[ 40 ];
     int i_fd, i_port;
@@ -160,52 +174,43 @@ int network_ChannelJoin( int i_channel )
     struct timeval delay;
     fd_set fds;
 
-    if( p_main->p_channel->i_channel == i_channel )
+    if( p_this->p_vlc->p_channel == NULL )
+    {
+        msg_Warn( p_this, "channels not initialized" );
+        return 0;
+    }
+
+    if( p_this->p_vlc->p_channel->i_channel == i_channel )
     {
         return 0;
     }
 
-    /* FIXME: ugly hack to close input and outputs */
-    if( p_input_bank->pp_input[0] != NULL )
+    if( !config_GetInt( p_this, "network-channel" ) )
     {
-        /* end playing item */
-        p_input_bank->pp_input[0]->b_eof = 1;
-
-        /* update playlist */
-        vlc_mutex_lock( &p_main->p_playlist->change_lock );
-
-        p_main->p_playlist->i_index--;
-        p_main->p_playlist->b_stopped = 1;
-
-        vlc_mutex_unlock( &p_main->p_playlist->change_lock );
-    }
-
-    if( !config_GetIntVariable( "network-channel" ) )
-    {
-        intf_ErrMsg( "network: channels disabled, to enable them, use the "
-                     "--channels option" );
+        msg_Err( p_this, "channels disabled, to enable them, use the"
+                         " --channels option" );
         return -1;
     }
 
-    if( config_GetIntVariable( "ipv4" ) )
+    if( config_GetInt( p_this, "ipv4" ) )
     {
         psz_network = "ipv4";
     }
-    if( config_GetIntVariable( "ipv6" ) )
+    if( config_GetInt( p_this, "ipv6" ) )
     {
         psz_network = "ipv6";
     }
 
     /* Getting information about the channel server */
-    if( !(psz_vlcs = config_GetPszVariable( "channel-server" )) )
+    if( !(psz_vlcs = config_GetPsz( p_this, "channel-server" )) )
     {
-        intf_ErrMsg( "network: configuration variable channel_server empty" );
+        msg_Err( p_this, "configuration variable channel-server empty" );
         return -1;
     }
 
-    i_port = config_GetIntVariable( "channel-port" );
+    i_port = config_GetInt( p_this, "channel-port" );
 
-    intf_WarnMsg( 5, "channel: connecting to %s:%d", psz_vlcs, i_port );
+    msg_Dbg( p_this, "connecting to %s:%d", psz_vlcs, i_port );
 
     /* Prepare the network_socket_t structure */
     socket_desc.i_type = NETWORK_UDP;
@@ -215,26 +220,26 @@ int network_ChannelJoin( int i_channel )
     socket_desc.i_server_port = i_port;
 
     /* Find an appropriate network module */
-    p_network = module_Need( MODULE_CAPABILITY_NETWORK, psz_network,
-                             &socket_desc );
+    p_this->p_private = (void*) &socket_desc;
+    p_network = module_Need( p_this, "network", psz_network/*, &socket_desc*/ );
     if( p_network == NULL )
     {
         return( -1 );
     }
-    module_Unneed( p_network );
+    module_Unneed( p_this, p_network );
 
     free( psz_vlcs ); /* Do we really need this ? -- Meuuh */
     i_fd = socket_desc.i_handle;
 
     /* Look for the interface MAC address */
-    if( GetMacAddress( i_fd, psz_mac ) )
+    if( GetMacAddress( p_this, i_fd, psz_mac ) )
     {
-        intf_ErrMsg( "network error: failed getting MAC address" );
+        msg_Err( p_this, "failed getting MAC address" );
         close( i_fd );
         return -1;
     }
 
-    intf_WarnMsg( 6, "network: MAC address is %s", psz_mac );
+    msg_Dbg( p_this, "MAC address is %s", psz_mac );
 
     /* Build the message */
     sprintf( psz_mess, "%d %u %lu %s \n", i_channel, VLCS_VERSION,
@@ -244,11 +249,11 @@ int network_ChannelJoin( int i_channel )
     /* Send the message */
     send( i_fd, psz_mess, MESSAGE_LENGTH, 0 );
 
-    intf_WarnMsg( 2, "network: attempting to join channel %d", i_channel );
+    msg_Dbg( p_this, "attempting to join channel %d", i_channel );
 
     /* We have changed channels ! (or at least, we tried) */
-    p_main->p_channel->last_change = mdate();
-    p_main->p_channel->i_channel = i_channel;
+    p_this->p_vlc->p_channel->last_change = mdate();
+    p_this->p_vlc->p_channel->i_channel = i_channel;
 
     /* Wait 5 sec for an answer from the server */
     delay.tv_sec = 5;
@@ -259,16 +264,14 @@ int network_ChannelJoin( int i_channel )
     switch( select( i_fd + 1, &fds, NULL, NULL, &delay ) )
     {
         case 0:
-            intf_ErrMsg( "network error: no answer from vlcs" );
+            msg_Err( p_this, "no answer from vlcs" );
             close( i_fd );
             return -1;
-            break;
 
         case -1:
-            intf_ErrMsg( "network error: error while listening to vlcs" );
+            msg_Err( p_this, "error while listening to vlcs" );
             close( i_fd );
             return -1;
-            break;
     }
 
     recv( i_fd, psz_mess, MESSAGE_LENGTH, 0 );
@@ -276,43 +279,36 @@ int network_ChannelJoin( int i_channel )
 
     if( !strncasecmp( psz_mess, "E:", 2 ) )
     {
-        intf_ErrMsg( "network error: vlcs said '%s'", psz_mess + 2 );
+        msg_Err( p_this, "vlcs said '%s'", psz_mess + 2 );
         close( i_fd );
         return -1;
     }
     else if( !strncasecmp( psz_mess, "I:", 2 ) )
     {
-        intf_WarnMsg( 2, "network info: vlcs said '%s'", psz_mess + 2 );
+        msg_Dbg( p_this, "vlcs said '%s'", psz_mess + 2 );
     }
-    else /* We got something to play ! FIXME: not very nice */
+    else
     {
-#   define p_item \
-        (&p_main->p_playlist->p_item[ p_main->p_playlist->i_index + 1])
-        vlc_mutex_lock( &p_main->p_playlist->change_lock );
-        /* FIXME 2 */
-        p_main->p_playlist->b_stopped = 0;
-        if( p_item )
+        /* We got something to play ! */
+        playlist_t *p_playlist;
+        p_playlist = vlc_object_find( p_this, VLC_OBJECT_PLAYLIST,
+                                              FIND_ANYWHERE );
+        if( p_playlist != NULL )
         {
-            free( p_item->psz_name );
-            p_item->psz_name = strdup( psz_mess );
-            /* Unlock _afterwards_ */
-            vlc_mutex_unlock( &p_main->p_playlist->change_lock );
-        }
-        else
-        {
-            /* Unlock _before_ */
-            vlc_mutex_unlock( &p_main->p_playlist->change_lock );
-            intf_PlaylistAdd( p_main->p_playlist, 0, psz_mess );
+            playlist_Add( p_playlist, psz_mess,
+                          PLAYLIST_APPEND | PLAYLIST_GO, PLAYLIST_END );
+            vlc_object_release( p_playlist );
         }
     }
 
     /* Close the socket and return nicely */
-#ifndef WIN32
-    close( i_fd );
-#else
+#   if defined( WIN32 )
     closesocket( i_fd );
-#endif
+#   else
+    close( i_fd );
+#   endif
 
+#endif
     return 0;
 }
 
@@ -321,7 +317,7 @@ int network_ChannelJoin( int i_channel )
 /*****************************************************************************
  * GetMacAddress: extract the MAC Address
  *****************************************************************************/
-static int GetMacAddress( int i_fd, char *psz_mac )
+static int GetMacAddress( vlc_object_t *p_this, int i_fd, char *psz_mac )
 {
 #if defined( SYS_LINUX )
     struct ifreq interface;
@@ -332,9 +328,9 @@ static int GetMacAddress( int i_fd, char *psz_mac )
      * Looking for information about the eth0 interface
      */
     interface.ifr_addr.sa_family = AF_INET;
-    if( !(psz_interface = config_GetPszVariable( "iface" )) )
+    if( !(psz_interface = config_GetPsz( p_this, "iface" )) )
     {
-        intf_ErrMsg( "network error: configuration variable iface empty" );
+        msg_Err( p_this, "configuration variable iface empty" );
         return -1;
     }
     strcpy( interface.ifr_name, psz_interface );
@@ -344,7 +340,7 @@ static int GetMacAddress( int i_fd, char *psz_mac )
 
     if( i_ret )
     {
-        intf_ErrMsg( "network error: ioctl SIOCGIFHWADDR failed" );
+        msg_Err( p_this, "ioctl SIOCGIFHWADDR failed" );
         return( i_ret );
     }
 
@@ -358,14 +354,55 @@ static int GetMacAddress( int i_fd, char *psz_mac )
 
     return( 0 );
 
-#elif defined( WIN32 )
+#elif defined( SYS_DARWIN )
+    char *psz_interface;
+    io_object_t ctrl_service;
+    CFTypeRef cfd_mac_address;
+    UInt8 ui_mac_address[kIOEthernetAddressSize];
+
+    if( !(psz_interface = config_GetPsz( p_this, "iface" )) )
+    {
+        msg_Err( p_this, "configuration variable iface empty" );
+        return( -1 );
+    }
+
+    if( GetNetIntfCtrl( psz_interface, &ctrl_service ) )
+    {
+        msg_Err( p_this, "GetNetIntfCtrl failed" );
+        return( -1 );
+    }
+
+    cfd_mac_address = IORegistryEntryCreateCFProperty( ctrl_service,
+                                                       CFSTR(kIOMACAddress),
+                                                       kCFAllocatorDefault,
+                                                       0 ); 
+    IOObjectRelease( ctrl_service );
+    if( cfd_mac_address == NULL )
+    {
+        msg_Err( p_this, "IORegistryEntryCreateCFProperty failed" );
+        return( -1 );
+    }
+
+    CFDataGetBytes( cfd_mac_address, 
+                    CFRangeMake(0, kIOEthernetAddressSize), 
+                    ui_mac_address );
+    CFRelease( cfd_mac_address );
+
+    sprintf( psz_mac, "%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x",
+                      ui_mac_address[0], ui_mac_address[1],
+                      ui_mac_address[2], ui_mac_address[3],
+                      ui_mac_address[4], ui_mac_address[5] ); 
+
+    return( 0 );
+
+#elif defined( WIN32 ) && !defined( UNDER_CE )
     int i, i_ret = -1;
 
     /* Get adapter list - support for more than one adapter */
     LANA_ENUM AdapterList;
     NCB       Ncb;
 
-    intf_WarnMsg( 2, "network: looking for MAC address" );
+    msg_Dbg( p_this, "looking for MAC address" );
 
     memset( &Ncb, 0, sizeof( NCB ) );
     Ncb.ncb_command = NCBENUM;
@@ -391,7 +428,51 @@ static int GetMacAddress( int i_fd, char *psz_mac )
 #endif
 }
 
-#ifdef WIN32
+#ifdef SYS_DARWIN
+/*****************************************************************************
+ * GetNetIntfCtrl : get parent controller for network interface 
+ *****************************************************************************/
+static int GetNetIntfCtrl( const char *psz_interface,
+                           io_object_t *ctrl_service )
+{
+    mach_port_t port;
+    kern_return_t ret;
+    io_object_t intf_service;
+    io_iterator_t intf_iterator;
+
+    /* get port for IOKit communication */
+    if( ( ret = IOMasterPort( MACH_PORT_NULL, &port ) ) != KERN_SUCCESS )
+    {
+        return( -1 );
+    }
+
+    /* look up the IOService object for the interface */
+    ret = IOServiceGetMatchingServices( port, IOBSDNameMatching( port, 0, 
+                                        psz_interface ), &intf_iterator ); 
+    if( ret != KERN_SUCCESS )
+    {
+        return( -1 );
+    }
+
+    intf_service = IOIteratorNext( intf_iterator );
+    if( intf_service == NULL )
+    {
+        return( -1 );
+    }
+
+    ret = IORegistryEntryGetParentEntry( intf_service,
+                                         kIOServicePlane,
+                                         ctrl_service );
+    IOObjectRelease( intf_service );
+    if( ret != KERN_SUCCESS )
+    {
+        return( -1 );
+    }
+
+    return( 0 );
+}
+
+#elif defined( WIN32 ) && !defined( UNDER_CE )
 /*****************************************************************************
  * GetAdapterInfo : gets some informations about the interface using NETBIOS
  *****************************************************************************/
@@ -411,7 +492,7 @@ static int GetAdapterInfo( int i_adapter, char *psz_string )
 
     if( Netbios( &Ncb ) != NRC_GOODRET )
     {
-        intf_ErrMsg( "network error: reset returned %i", Ncb.ncb_retcode );
+//X        intf_ErrMsg( "network error: reset returned %i", Ncb.ncb_retcode );
         return -1;
     }
 
@@ -438,13 +519,13 @@ static int GetAdapterInfo( int i_adapter, char *psz_string )
                 (int) ( Adapter.adapt.adapter_address[4] ),
                 (int) ( Adapter.adapt.adapter_address[5] ) );
 
-        intf_WarnMsg( 2, "network: found MAC address %s", psz_string );
+//X        intf_WarnMsg( 2, "network: found MAC address %s", psz_string );
 
         return 0;
     }
     else
     {
-        intf_ErrMsg( "network error: ASTAT returned %i", Ncb.ncb_retcode );
+//X        intf_ErrMsg( "network error: ASTAT returned %i", Ncb.ncb_retcode );
         return -1;
     }
 }
